@@ -36,6 +36,7 @@
 #include <bits/std_istream.h>
 #include <bits/std_ostream.h>
 #include <bits/std_vector.h>
+#include <bits/std_memory.h>      // for auto_ptr
 
 namespace std {
 
@@ -51,58 +52,46 @@ namespace std {
 
   locale::_Impl::_Impl(size_t __numfacets, size_t __refs)
   : _M_num_references(__refs - 1)
-    //  , _M_facets (numfacets, (facet*)0)
-    //  , _M_category_names (_S_num_categories, string ("*"))
+  , _M_facets(0)
+  , _M_category_names(0)
   , _M_has_name(false)
   , _M_cached_name_ok(false)
   , _M_cached_name(string ("*"))
   { 
     typedef vector<facet*, __STL_DEFAULT_ALLOCATOR(facet*) > __vec_facet;
     typedef vector<string, __STL_DEFAULT_ALLOCATOR(string) > __vec_string;
-    try {
-      _M_facets = new __vec_facet(__numfacets, (facet*)0);
-    }
-    catch (...) {
-      delete _M_facets;
-      throw;
-    }
-    try {
-       _M_category_names = new __vec_string(_S_num_categories, string("*"));
-    }
-    catch (...) {
-      delete _M_category_names;
-      throw;
-    }
+
+    auto_ptr<__vec_facet> __pvf(new __vec_facet(__numfacets, (facet*)0));
+    auto_ptr<__vec_string> __pcn(new __vec_string(_S_num_categories,
+						  string("*")));
+    _M_facets = __pvf.release();
+    _M_category_names = __pcn.release();
   }
   
   locale::_Impl::_Impl(const _Impl& __other, size_t __refs)
   : _M_num_references(__refs)
-    //  , _M_facets (other._M_facets)
-    //  , _M_category_names (other._M_category_names)
+  , _M_facets(0)
+  , _M_category_names(0)
   , _M_has_name(__other._M_has_name)
   , _M_cached_name_ok(__other._M_cached_name_ok)
   , _M_cached_name(__other._M_cached_name)
   {
     typedef vector<facet*, __STL_DEFAULT_ALLOCATOR(facet*) > __vec_facet;
     typedef vector<string, __STL_DEFAULT_ALLOCATOR(string) > __vec_string;
-    try {
-      _M_facets = new __vec_facet(*(__other._M_facets));
-    }
-    catch (...) {
-      delete _M_facets;
-      throw;
-    }
-    try {
-       _M_category_names = new __vec_string(*(__other._M_category_names));
-    }
-    catch (...) {
-      delete _M_category_names;
-      throw;
-    }
 
-    std::vector<facet*>::iterator __it = _M_facets->begin();
-    for (; __it != _M_facets->end(); ++__it)
+    auto_ptr<__vec_facet> __pvf(new __vec_facet(*(__other._M_facets)));
+    auto_ptr<__vec_string> 
+      __pcn(new __vec_string(*(__other._M_category_names)));
+
+    std::vector<facet*>::iterator __it = __pvf->begin();
+    for (; __it != __pvf->end(); ++__it)
       (*__it)->_M_add_reference();
+
+    // these must be last since in the presence of an exception, the 
+    // destructor for 'this' won't run until AFTER execution has passed  
+    // the closing brace of the constructor
+    _M_facets = __pvf.release();
+    _M_category_names = __pcn.release();
   }
   
   void
@@ -302,7 +291,6 @@ namespace std {
     return *this;
   }
 
-  ///////
   locale
   locale::global(const locale& __other)
   {
@@ -317,7 +305,6 @@ namespace std {
     return __keep;
   }
 
-  /////////
   string
   locale::name() const
   {
@@ -325,7 +312,6 @@ namespace std {
     return "*";
   }
 
-  ///////
   locale const&
   locale::classic()
   {
@@ -455,27 +441,17 @@ namespace std {
   size_t locale::id::_S_highwater;  // init'd to 0 by linker
 
 
+  // Platform-specific initialization code for ctype tables.
+  #include <ctype.cc>
+
   locale::id ctype<char>::id;
 
   ctype<char>::~ctype()
   { if (_M_del) delete[] table (); }
 
-#if defined(__GLIBC__) && (__GLIBC__ >= 2) && defined(_GLIBCPP_USE_CTYPE_ISBIT)
-  // XXX these are Glibc-specific.
-  const int* const& ctype<char>::_S_toupper = __ctype_toupper;
-  const int* const& ctype<char>::_S_tolower = __ctype_tolower;
-  const ctype_base::mask* const& ctype<char>::_S_table = __ctype_b;
-#else
-  const int* const& ctype<char>::_S_toupper = 0;
-  const int* const& ctype<char>::_S_tolower = 0;
-  const ctype_base::mask* const& ctype<char>::_S_table = 0;
-#endif
-  
   char
   ctype<char>::do_toupper(char __c) const
-  {
-    return _S_toupper[(int) __c];
-  }
+  { return _S_toupper[(int) __c]; }
 
   const char*
   ctype<char>::do_toupper(char* __low, const char* __high) const
@@ -490,9 +466,7 @@ namespace std {
 
   char
   ctype<char>::do_tolower(char __c) const
-  {
-    return _S_tolower[(int) __c];
-  }
+  { return _S_tolower[(int) __c]; }
 
   const char* 
   ctype<char>::do_tolower(char* __low, const char* __high) const
@@ -507,9 +481,7 @@ namespace std {
 
   char
   ctype<char>::do_widen(char __c) const
-  {
-    return __c;
-  }
+  { return __c; }
   
   const char* 
   ctype<char>::
@@ -521,9 +493,7 @@ namespace std {
   
   char
   ctype<char>::do_narrow(char __c, char /*__dfault*/) const
-  {
-    return __c;
-  }
+  { return __c; }
   
   const char* 
   ctype<char>::
@@ -534,11 +504,9 @@ namespace std {
     return __high;
   }
 
-  template<>
-    ctype_byname<char>::ctype_byname(const char* /*__s*/, size_t __refs)
-    : ctype<char> (new mask[table_size], true, __refs)
-    { }
-  
+  ctype_byname<char>::ctype_byname(const char* /*__s*/, size_t __refs)
+  : ctype<char> (new mask[table_size], true, __refs)
+  { }
 
   locale::id codecvt<char, char, mbstate_t>::id;
 
@@ -641,9 +609,7 @@ namespace std {
   
   string
   collate<char>::do_transform(const char* __lo, const char* __hi) const
-  {
-    return string(__lo, __hi - __lo);
-  }
+  { return string(__lo, __hi - __lo); }
   
   long
   collate<char>::do_hash(const char* __lo, const char* __hi) const
@@ -655,48 +621,31 @@ namespace std {
     return __val;
   }
   
-  template<>
-    collate_byname<char>::collate_byname(const char* /*__s*/, size_t __refs)
-    : collate<char> (__refs)
-    { }
+  collate_byname<char>::collate_byname(const char* /*__s*/, size_t __refs)
+  : collate<char> (__refs)
+  { }
 
-  template<>
-    numpunct_byname<char>::numpunct_byname(const char* /*__s*/, size_t __refs)
-    : numpunct<char> (__refs)
-    { }
+  numpunct_byname<char>::numpunct_byname(const char* /*__s*/, size_t __refs)
+  : numpunct<char> (__refs)
+  { }
 
-
-  template<>
-    moneypunct_byname<char, false>::
-    moneypunct_byname(const char* /*__s*/, size_t __refs)
-    : moneypunct<char, false> (__refs)
-    { }
+  moneypunct_byname<char, false>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<char, false> (__refs)
+  { }
   
-  template<>
-    moneypunct_byname<char, true>::
-    moneypunct_byname(const char* /*__s*/, size_t __refs)
-    : moneypunct<char, true> (__refs)
-    { }
+  moneypunct_byname<char, true>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<char, true> (__refs)
+  { }
   
-  template<>
-    messages_byname<char>::messages_byname(const char* /*__s*/, size_t __refs)
-    : messages<char> (__refs)
-    { }
+  messages_byname<char>::messages_byname(const char* /*__s*/, size_t __refs)
+  : messages<char> (__refs)
+  { }
 
 #ifdef _GLIBCPP_USE_WCHAR_T  
   locale::id ctype<wchar_t>::id;
 
-#if defined(__GLIBC__) && (__GLIBC__ >= 2) && defined(_GLIBCPP_USE_CTYPE_ISBIT)
-  // XXX These are Glibc-specific.
-  const int* const& ctype<wchar_t>::_S_toupper = __ctype_toupper;
-  const int* const& ctype<wchar_t>::_S_tolower = __ctype_tolower;
-  const ctype_base::mask* const& ctype<wchar_t>::_S_table = __ctype_b;
-#else
-  const int* const& ctype<wchar_t>::_S_toupper = 0;
-  const int* const& ctype<wchar_t>::_S_tolower = 0;
-  const ctype_base::mask* const& ctype<wchar_t>::_S_table = 0;
-#endif
-
   ctype<wchar_t>::ctype(size_t __refs)
   : _Ctype<wchar_t>(__refs)
   { }
@@ -705,9 +654,7 @@ namespace std {
 
   bool
   ctype<wchar_t>::do_is(mask __m, char_type __c) const
-  {
-    return (__c < _S_table_size) && (_S_table[__c] & __m); 
-  }
+  { return (__c < _S_table_size) && (_S_table[__c] & __m); }
   
   const wchar_t* 
   ctype<wchar_t>::do_is(const wchar_t* __low, const wchar_t* __high,
@@ -741,9 +688,7 @@ namespace std {
 
   wchar_t
   ctype<wchar_t>::do_toupper(char_type __c) const
-  {
-    return (__c < _S_table_size) ? _S_toupper[__c] : __c;
-  }
+  { return (__c < _S_table_size) ? _S_toupper[__c] : __c; }
   
   const wchar_t*
   ctype<wchar_t>::do_toupper(char_type* low, const char_type* high) const
@@ -756,9 +701,7 @@ namespace std {
   
   wchar_t
   ctype<wchar_t>::do_tolower(char_type __c) const
-  {
-    return (__c < _S_table_size) ? _S_tolower[__c] : __c;
-  }
+  { return (__c < _S_table_size) ? _S_tolower[__c] : __c; }
   
   const wchar_t*
   ctype<wchar_t>::do_tolower(char_type* __low, const char_type* __high) const
@@ -771,9 +714,7 @@ namespace std {
 
   wchar_t
   ctype<wchar_t>::do_widen(char __c) const
-  {
-    return static_cast<wchar_t> ((unsigned char)__c);
-  }
+  { return static_cast<wchar_t> ((unsigned char)__c); }
   
   const char* 
   ctype<wchar_t>::do_widen(const char* __low, const char* __high,
@@ -786,9 +727,7 @@ namespace std {
 
   char
   ctype<wchar_t>::do_narrow(wchar_t __c, char __dfault) const
-  {
-    return (__c < _S_table_size) ? static_cast<char> (__c) : __dfault;
-  }
+  { return (__c < _S_table_size) ? static_cast<char> (__c) : __dfault; }
 
   const wchar_t*
   ctype<wchar_t>::do_narrow(const wchar_t* __low, const wchar_t* __high,
@@ -799,10 +738,9 @@ namespace std {
     return __high;
   }
 
-  template<>
-    ctype_byname<wchar_t>::ctype_byname(const char* /*__s*/, size_t __refs)
-    : ctype<wchar_t> (__refs)
-    { }
+  ctype_byname<wchar_t>::ctype_byname(const char* /*__s*/, size_t __refs)
+  : ctype<wchar_t> (__refs)
+  { }
 
   locale::id codecvt<wchar_t, char, mbstate_t>::id;
 
@@ -901,36 +839,33 @@ namespace std {
     return 0; // XXX not done
   }
 
-  template<>
-    numpunct_byname<wchar_t>::numpunct_byname(const char* /*__s*/,
-                                              size_t __refs)
-    : numpunct<wchar_t> (__refs)
-    { }
+  numpunct_byname<wchar_t>::numpunct_byname(const char* /*__s*/, size_t __refs)
+  : numpunct<wchar_t> (__refs)
+  { }
 
-  template<>
-    collate_byname<wchar_t>::collate_byname(const char* /*__s*/, size_t __refs)
-    : collate<wchar_t> (__refs)
-    { }
+  collate_byname<wchar_t>::collate_byname(const char* /*__s*/, size_t __refs)
+  : collate<wchar_t> (__refs)
+  { }
   
-  template<>
-    moneypunct_byname<wchar_t, false>::
-    moneypunct_byname(const char* /*__s*/, size_t __refs)
-    : moneypunct<wchar_t, false> (__refs)
-    { }
+  moneypunct_byname<wchar_t, false>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<wchar_t, false> (__refs)
+  { }
   
-  template<>
-    moneypunct_byname<wchar_t, true>::
-    moneypunct_byname(const char* /*__s*/, size_t __refs)
-    : moneypunct<wchar_t, true> (__refs)
-    { }
+  moneypunct_byname<wchar_t, true>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<wchar_t, true> (__refs)
+  { }
     
-  template<>
-    messages_byname<wchar_t>::messages_byname(const char* /*__s*/,
-                                              size_t __refs)
-    : messages<wchar_t> (__refs)
-    { }
+  messages_byname<wchar_t>::messages_byname(const char* /*__s*/, size_t __refs)
+  : messages<wchar_t> (__refs)
+  { }
 #endif //  _GLIBCPP_USE_WCHAR_T
 
 } // namespace std
 
 
+
+
+
+
