@@ -20,7 +20,7 @@
  * provided that the above copyright notice appear in all copies and
  * that both that copyright notice and this permission notice appear
  * in supporting documentation.  Silicon Graphics makes no
- * representations about the suitability of this software for any
+ * representations about the suitability of this  software for any
  * purpose.  It is provided "as is" without express or implied warranty.
  */
 
@@ -213,13 +213,25 @@ public:
   size_type max_size() const
     { return size_type(-1) / sizeof(_Tp); }
   size_type capacity() const
-    { return size_type(_M_end_of_storage - begin()); }
+    { return size_type(const_iterator(_M_end_of_storage) - begin()); }
   bool empty() const
     { return begin() == end(); }
 
   reference operator[](size_type __n) { return *(begin() + __n); }
   const_reference operator[](size_type __n) const { return *(begin() + __n); }
 
+#ifdef __STL_THROW_RANGE_ERRORS
+  void _M_range_check(size_type __n) const {
+    if (__n >= this->size())
+      __stl_throw_range_error("vector");
+  }
+
+  reference at(size_type __n)
+    { _M_range_check(__n); return (*this)[__n]; }
+  const_reference at(size_type __n) const
+    { _M_range_check(__n); return (*this)[__n]; }
+#endif /* __STL_THROW_RANGE_ERRORS */
+
   explicit vector(const allocator_type& __a = allocator_type())
     : _Base(__a) {}
 
@@ -271,7 +283,7 @@ public:
   void reserve(size_type __n) {
     if (capacity() < __n) {
       const size_type __old_size = size();
-      iterator __tmp(_M_allocate_and_copy(__n, _M_start, _M_finish));
+      pointer __tmp = _M_allocate_and_copy(__n, _M_start, _M_finish);
       destroy(_M_start, _M_finish);
       _M_deallocate(_M_start, _M_end_of_storage - _M_start);
       _M_start = __tmp;
@@ -285,7 +297,8 @@ public:
   // The range version is a member template, so we dispatch on whether
   // or not the type is an integer.
 
-  void assign(size_type __n, const _Tp& __val);
+  void assign(size_type __n, const _Tp& __val) { _M_fill_assign(__n, __val); }
+  void _M_fill_assign(size_type __n, const _Tp& __val);
 
 #ifdef __STL_MEMBER_TEMPLATES
   
@@ -297,7 +310,7 @@ public:
 
   template <class _Integer>
   void _M_assign_dispatch(_Integer __n, _Integer __val, __true_type)
-    { assign((size_type) __n, (_Tp) __val); }
+    { _M_fill_assign((size_type) __n, (_Tp) __val); }
 
   template <class _InputIter>
   void _M_assign_dispatch(_InputIter __first, _InputIter __last, __false_type)
@@ -347,7 +360,7 @@ public:
       ++_M_finish;
     }
     else
-      _M_insert_aux(__position, __x);
+      _M_insert_aux(iterator(__position), __x);
     return begin() + __n;
   }
   iterator insert(iterator __position) {
@@ -357,7 +370,7 @@ public:
       ++_M_finish;
     }
     else
-      _M_insert_aux(__position);
+      _M_insert_aux(iterator(__position));
     return begin() + __n;
   }
 #ifdef __STL_MEMBER_TEMPLATES
@@ -370,9 +383,8 @@ public:
 
   template <class _Integer>
   void _M_insert_dispatch(iterator __pos, _Integer __n, _Integer __val,
-                          __true_type) {
-    insert(__pos, (size_type) __n, (_Tp) __val);
-  }
+                          __true_type)
+    { _M_fill_insert(__pos, (size_type) __n, (_Tp) __val); }
 
   template <class _InputIterator>
   void _M_insert_dispatch(iterator __pos,
@@ -385,7 +397,10 @@ public:
               const_iterator __first, const_iterator __last);
 #endif /* __STL_MEMBER_TEMPLATES */
 
-  void insert (iterator __pos, size_type __n, const _Tp& __x);
+  void insert (iterator __pos, size_type __n, const _Tp& __x)
+    { _M_fill_insert(__pos, __n, __x); }
+
+  void _M_fill_insert (iterator __pos, size_type __n, const _Tp& __x);
 
   void pop_back() {
     --_M_finish;
@@ -418,10 +433,10 @@ protected:
 
 #ifdef __STL_MEMBER_TEMPLATES
   template <class _ForwardIterator>
-  iterator _M_allocate_and_copy(size_type __n, _ForwardIterator __first, 
+  pointer _M_allocate_and_copy(size_type __n, _ForwardIterator __first, 
                                                _ForwardIterator __last)
 {
-    iterator __result(_M_allocate(__n));
+    pointer __result = _M_allocate(__n);
     __STL_TRY {
       uninitialized_copy(__first, __last, __result);
       return __result;
@@ -429,7 +444,7 @@ protected:
     __STL_UNWIND(_M_deallocate(__result, __n));
   }
 #else /* __STL_MEMBER_TEMPLATES */
-  iterator _M_allocate_and_copy(size_type __n, const_iterator __first, 
+  pointer _M_allocate_and_copy(size_type __n, const_iterator __first, 
                                                const_iterator __last)
   {
     iterator __result(_M_allocate(__n));
@@ -533,7 +548,7 @@ vector<_Tp,_Alloc>::operator=(const vector<_Tp, _Alloc>& __x)
   if (&__x != this) {
     const size_type __xlen = __x.size();
     if (__xlen > capacity()) {
-      iterator __tmp(_M_allocate_and_copy(__xlen, __x.begin(), __x.end()));
+      pointer __tmp = _M_allocate_and_copy(__xlen, __x.begin(), __x.end());
       destroy(_M_start, _M_finish);
       _M_deallocate(_M_start, _M_end_of_storage - _M_start);
       _M_start = __tmp;
@@ -541,7 +556,7 @@ vector<_Tp,_Alloc>::operator=(const vector<_Tp, _Alloc>& __x)
     }
     else if (size() >= __xlen) {
       iterator __i(copy(__x.begin(), __x.end(), begin()));
-      destroy(__i, _M_finish);
+      destroy(__i.base(), _M_finish);
     }
     else {
       copy(__x.begin(), __x.begin() + size(), _M_start);
@@ -553,7 +568,8 @@ vector<_Tp,_Alloc>::operator=(const vector<_Tp, _Alloc>& __x)
 }
 
 template <class _Tp, class _Alloc>
-void vector<_Tp, _Alloc>::assign(size_t __n, const value_type& __val) {
+void vector<_Tp, _Alloc>::_M_fill_assign(size_t __n, const value_type& __val) 
+{
   if (__n > capacity()) {
     vector<_Tp, _Alloc> __tmp(__n, __val, get_allocator());
     __tmp.swap(*this);
@@ -596,7 +612,7 @@ vector<_Tp, _Alloc>::_M_assign_aux(_ForwardIter __first, _ForwardIter __last,
   }
   else if (size() >= __len) {
     iterator __new_finish(copy(__first, __last, _M_start));
-    destroy(__new_finish, _M_finish);
+    destroy(__new_finish.base(), _M_finish);
     _M_finish = __new_finish;
   }
   else {
@@ -650,19 +666,22 @@ vector<_Tp, _Alloc>::_M_insert_aux(iterator __position)
   if (_M_finish != _M_end_of_storage) {
     construct(_M_finish, *(_M_finish - 1));
     ++_M_finish;
-    copy_backward(__position, _M_finish - 2, _M_finish - 1);
+    copy_backward(__position, iterator(_M_finish - 2), 
+		  iterator(_M_finish - 1));
     *__position = _Tp();
   }
   else {
     const size_type __old_size = size();
     const size_type __len = __old_size != 0 ? 2 * __old_size : 1;
-    iterator __new_start(_M_allocate(__len));
-    iterator __new_finish(__new_start);
+    pointer __new_start = _M_allocate(__len);
+    pointer __new_finish = __new_start;
     __STL_TRY {
-      __new_finish = uninitialized_copy(_M_start, __position, __new_start);
+      __new_finish = uninitialized_copy(iterator(_M_start), __position, 
+					__new_start);
       construct(__new_finish);
       ++__new_finish;
-      __new_finish = uninitialized_copy(__position, _M_finish, __new_finish);
+      __new_finish = uninitialized_copy(__position, iterator(_M_finish), 
+					__new_finish);
     }
     __STL_UNWIND((destroy(__new_start,__new_finish), 
                   _M_deallocate(__new_start,__len)));
@@ -675,8 +694,8 @@ vector<_Tp, _Alloc>::_M_insert_aux(iterator __position)
 }
 
 template <class _Tp, class _Alloc>
-void vector<_Tp, _Alloc>::insert(iterator __position, size_type __n, 
-                                 const _Tp& __x)
+void vector<_Tp, _Alloc>::_M_fill_insert(iterator __position, size_type __n, 
+                                         const _Tp& __x)
 {
   if (__n != 0) {
     if (size_type(_M_end_of_storage - _M_finish) >= __n) {
@@ -745,7 +764,7 @@ vector<_Tp, _Alloc>::_M_range_insert(iterator __position,
     size_type __n = 0;
     distance(__first, __last, __n);
     if (size_type(_M_end_of_storage - _M_finish) >= __n) {
-      const size_type __elems_after = _M_finish - __position;
+      const size_type __elems_after = _M_finish - __position.base();
       iterator __old_finish(_M_finish);
       if (__elems_after > __n) {
         uninitialized_copy(_M_finish - __n, _M_finish, _M_finish);
@@ -769,18 +788,19 @@ vector<_Tp, _Alloc>::_M_range_insert(iterator __position,
       iterator __new_start(_M_allocate(__len));
       iterator __new_finish(__new_start);
       __STL_TRY {
-        __new_finish = uninitialized_copy(_M_start, __position, __new_start);
+        __new_finish = uninitialized_copy(iterator(_M_start), 
+					  __position, __new_start);
         __new_finish = uninitialized_copy(__first, __last, __new_finish);
         __new_finish
-          = uninitialized_copy(__position, _M_finish, __new_finish);
+          = uninitialized_copy(__position, iterator(_M_finish), __new_finish);
       }
       __STL_UNWIND((destroy(__new_start,__new_finish), 
-                    _M_deallocate(__new_start,__len)));
+                    _M_deallocate(__new_start.base(),__len)));
       destroy(_M_start, _M_finish);
       _M_deallocate(_M_start, _M_end_of_storage - _M_start);
-      _M_start = __new_start;
-      _M_finish = __new_finish;
-      _M_end_of_storage = __new_start + __len;
+      _M_start = __new_start.base();
+      _M_finish = __new_finish.base();
+      _M_end_of_storage = __new_start.base() + __len;
     }
   }
 }
